
import datetime
import numpy as np
import matplotlib.pyplot as plt
import logging
from matplotlib import pyplot as plt
import random

classify_dict = np.load("classify.dict.npz")['arr_0'][()]
train_day15 = []

def fitFunc(position, allexample):
    basesum = 0.0
    for i in allexample:
        new_temp = classify_dict[i[0]].reshape(1, 15)
        pred_new = i[1] + np.dot(np.dot(new_temp, position), new_temp.T)
        basesum += abs(pred_new[0][0] - i[2]) / i[2]
    return basesum/len(allexample)

class bird:
    def __init__(self, speed, position, fit, lBestPosition, lBestFit):
        self.speed = speed
        self.position = position
        self.fit = fit
        self.lBestFit = lBestFit
        self.lBestPosition = lBestPosition

class PSO:
    def __init__(self, fitFunc, birdNum, c1, c2, solutionSpace, extend, birds):
        self.fitFunc = fitFunc
        self.c1 = c1
        self.c2 = c2
        self.w = 0.9
        self.extend = extend
        self.solutionmax = 100
        self.solutionmin = -100
        if birds == None:
            self.birds, self.best = self.initbirds(birdNum, solutionSpace)
        else:
            self.best = birds[0]
            for birdi in birds:
                if birdi.fit < self.best.fit:
                    self.best = birdi
            self.birds = birds

    def initbirds(self, size, solutionSpace):
        birds = []
        for i in range(size):
            position = (np.random.random(size=(15,15))-0.5)*solutionSpace
            speed = np.zeros(shape=(15, 15))
            fit = self.fitFunc(position, self.extend)
            birds.append(bird(speed=speed, position=position, fit=fit, lBestPosition=position, lBestFit=fit))
        best = birds[0]
        for birdi in birds:
            if birdi.fit < best.fit:
                best = birdi
        return birds,best

    def updateBirds(self):
        for birdi in self.birds:
            birdi.speed = self.w * birdi.speed + self.c1 * random.random() * (birdi.lBestPosition - birdi.position) + self.c2 * random.random() * (self.best.lBestPosition - birdi.position)
            birdi.position = birdi.position + birdi.speed
            for i in range(birdi.position.shape[0]):
                for j in range(birdi.position.shape[1]):
                    if birdi.position[i][j] > self.solutionmax:
                        birdi.position[i][j] = self.solutionmax*0.5 + self.solutionmax*random.random()*0.5
                    if birdi.position[i][j] < self.solutionmin:
                        birdi.position[i][j] = self.solutionmin*0.5 + self.solutionmin*random.random()*0.5
            birdi.fit = self.fitFunc(birdi.position, self.extend) 
            if birdi.fit < birdi.lBestFit:
                birdi.lBestFit = birdi.fit
                birdi.lBestPosition = birdi.position

    def solve(self, maxIter):
        for i in range(maxIter):
            self.w = 0.5*(maxIter-i)/maxIter + 0.4
            self.updateBirds()
            for birdi in self.birds:
                if birdi.fit < self.best.lBestFit:
                    self.best = birdi
            if self.best.lBestFit < 0.05:
                break
        return self.best.lBestPosition, self.birds, self.best.lBestFit
